Package: worldbank
Title: Client for the 'World Bank' APIs
Version: 0.10.0
Authors@R: 
    person("Maximilian", "Mücke", , "muecke.maximilian@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0009-0000-9432-9795"))
Description: Download and search data from the 'World Bank' APIs,
    including the 'Indicators' API, the 'Poverty and Inequality Platform
    (PIP)' API, the 'Finances One' API, and the 'Projects' API. See
    <https://datahelpdesk.worldbank.org/knowledgebase/articles/889386-developer-information-overview>
    for further details.
License: MIT + file LICENSE
URL: https://m-muecke.github.io/worldbank/,
        https://github.com/m-muecke/worldbank
BugReports: https://github.com/m-muecke/worldbank/issues
Depends: R (>= 4.1.0)
Imports: httr2 (>= 1.1.0), stats, tools
Suggests: ggplot2, scales, testthat (>= 3.3.0)
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.1.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-08-20 13:44:28 UTC; mmuecke
Author: Maximilian Mücke [aut, cre] (ORCID:
    <https://orcid.org/0009-0000-9432-9795>)
Maintainer: Maximilian Mücke <muecke.maximilian@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-21 05:40:02 UTC
