# error parsing works

    Code
      wb_indicator("something.wrong")
    Condition
      Error in `req_perform()`:
      ! HTTP 200 OK.
      Error code: 120
      The provided parameter value is not valid
      Read more at <https://datahelpdesk.worldbank.org/knowledgebase/articles/898620-api-error-codes>

