% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pip.R
\name{pip_group}
\alias{pip_group}
\title{Return aggregation of PIP statistics}
\source{
\url{https://pip.worldbank.org/api}
}
\usage{
pip_group(
  country = NULL,
  year = NULL,
  povline = 2.15,
  popshare = NULL,
  group_by = c("wb", "none"),
  fill_gaps = FALSE,
  welfare_type = c("all", "consumption", "income"),
  reporting_level = c("all", "national", "rural", "urban"),
  additional_ind = FALSE,
  release_version = NULL,
  ppp_version = NULL,
  version = NULL
)
}
\arguments{
\item{country}{(\code{NULL} | \code{character()})\cr
Countries for which statistics are to be computed, specified as ISO3 codes. Default \code{NULL}.}

\item{year}{(\code{NULL} | \code{character()} | \code{numeric()})\cr
Years for which statistics are to be computed, specified as YYYY. Default \code{NULL}.}

\item{povline}{(\code{NULL} | \code{numeric(1)})\cr
Poverty line to be used to compute poverty measures, between \code{0} and \code{2700}. Poverty lines
are only accepted up to 3 decimals. Default \code{2.15}.}

\item{popshare}{(\code{NULL} | \code{numeric(1)})\cr
Proportion of the population living below the poverty line, between \code{0} and \code{1}. Takes
precedence over \code{povline}: if both are supplied, the poverty line is derived from \code{popshare}.
Default \code{NULL}.}

\item{group_by}{(\code{character(1)})\cr
Aggregate results by pre-defined sub-groups. Default \code{"wb"}.}

\item{fill_gaps}{(\code{logical(1)})\cr
Whether to fill gaps in the data. Default \code{FALSE}.}

\item{welfare_type}{(\code{character(1)})\cr
Type of welfare measure to be used. Default \code{"all"}.}

\item{reporting_level}{(\code{character(1)})\cr
Level of reporting for the statistics. Default \code{"all"}.}

\item{additional_ind}{(\code{logical(1)})\cr
Whether to include additional indicators. Default \code{FALSE}.}

\item{release_version}{(\code{NULL} | \code{character(1)} | \code{numeric(1)})\cr
Version of the data release in YYYYMMDD format. Default \code{NULL}.}

\item{ppp_version}{(\code{NULL} | \code{character(1)} | \code{numeric(1)})\cr
Version of the data in YYYY format. Default \code{NULL}.}

\item{version}{(\code{NULL} | \code{character(1)})\cr
Version of the data. Default \code{NULL}.}
}
\value{
A \code{data.frame()} with the requested statistics.
}
\description{
Return aggregation of PIP statistics
}
\examples{
\dontshow{if (httr2::is_online()) withAutoprint(\{ # examplesIf}
\donttest{
grp <- pip_group(c("AFE", "LAC"))
head(grp)
}
\dontshow{\}) # examplesIf}
}
\seealso{
Other poverty and inequality statistics:
\code{\link[=pip_aux]{pip_aux()}},
\code{\link[=pip_citation]{pip_citation()}},
\code{\link[=pip_cp]{pip_cp()}},
\code{\link[=pip_data]{pip_data()}},
\code{\link[=pip_health_check]{pip_health_check()}},
\code{\link[=pip_info]{pip_info()}},
\code{\link[=pip_valid_params]{pip_valid_params()}},
\code{\link[=pip_versions]{pip_versions()}}
}
\concept{poverty and inequality statistics}
