% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pip.R
\name{pip_versions}
\alias{pip_versions}
\title{Return the available data versions}
\source{
\url{https://pip.worldbank.org/api}
}
\usage{
pip_versions()
}
\value{
A \code{data.frame()} with the available versions.
}
\description{
Return the available data versions
}
\examples{
\dontshow{if (httr2::is_online()) withAutoprint(\{ # examplesIf}
\donttest{
vers <- pip_versions()
head(vers)
}
\dontshow{\}) # examplesIf}
}
\seealso{
Other poverty and inequality statistics:
\code{\link[=pip_aux]{pip_aux()}},
\code{\link[=pip_citation]{pip_citation()}},
\code{\link[=pip_cp]{pip_cp()}},
\code{\link[=pip_data]{pip_data()}},
\code{\link[=pip_group]{pip_group()}},
\code{\link[=pip_health_check]{pip_health_check()}},
\code{\link[=pip_info]{pip_info()}},
\code{\link[=pip_valid_params]{pip_valid_params()}}
}
\concept{poverty and inequality statistics}
