% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/worldbank-package.R
\docType{package}
\name{worldbank-package}
\alias{worldbank}
\alias{worldbank-package}
\title{worldbank: Client for the 'World Bank' APIs}
\description{
Download and search data from the 'World Bank' APIs, including the 'Indicators' API, the 'Poverty and Inequality Platform (PIP)' API, the 'Finances One' API, and the 'Projects' API. See \url{https://datahelpdesk.worldbank.org/knowledgebase/articles/889386-developer-information-overview} for further details.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://m-muecke.github.io/worldbank/}
  \item \url{https://github.com/m-muecke/worldbank}
  \item Report bugs at \url{https://github.com/m-muecke/worldbank/issues}
}

}
\author{
\strong{Maintainer}: Maximilian Mücke \email{muecke.maximilian@gmail.com} (\href{https://orcid.org/0009-0000-9432-9795}{ORCID})

Authors:
\itemize{
  \item Maximilian Mücke \email{muecke.maximilian@gmail.com} (\href{https://orcid.org/0009-0000-9432-9795}{ORCID})
}

}
\keyword{internal}
