% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicators.R
\name{wb_indicator}
\alias{wb_indicator}
\title{World Bank indicator data}
\source{
\url{https://api.worldbank.org/v2/indicator}
}
\usage{
wb_indicator(indicator = NULL, lang = "en")
}
\arguments{
\item{indicator}{(\code{NULL} | \code{character(1)})\cr
Indicator to query. Default \code{NULL}. If \code{NULL}, all indicators are returned.}

\item{lang}{(\code{character(1)})\cr
Language to query. Default \code{"en"}.}
}
\value{
A \code{data.frame()} with the available indicators. The columns are:
\itemize{
\item \code{id}: The indicator ID.
\item \code{name}: The indicator name.
\item \code{unit}: The indicator unit.
\item \code{source_id}: The source ID.
\item \code{source_value}: The source value.
\item \code{source_note}: The source note.
\item \code{source_organization}: The source organization.
\item \code{topic_id}: The topic ID.
\item \code{topic_value}: The topic value.
}
}
\description{
List all indicators supported by the World Bank API.
}
\examples{
\dontshow{if (httr2::is_online()) withAutoprint(\{ # examplesIf}
\donttest{
wb_indicator("NY.GDP.MKTP.CD")
}
\dontshow{\}) # examplesIf}
}
\seealso{
Other indicators data:
\code{\link[=wb_bulk]{wb_bulk()}},
\code{\link[=wb_country]{wb_country()}},
\code{\link[=wb_data]{wb_data()}},
\code{\link[=wb_income_level]{wb_income_level()}},
\code{\link[=wb_language]{wb_language()}},
\code{\link[=wb_lending_type]{wb_lending_type()}},
\code{\link[=wb_region]{wb_region()}},
\code{\link[=wb_search]{wb_search()}},
\code{\link[=wb_source]{wb_source()}},
\code{\link[=wb_topic]{wb_topic()}}
}
\concept{indicators data}
